@@ -6,7 +6,8 @@ from collections import Counter
 from contextlib import contextmanager
 from copy import deepcopy
 from enum import Enum
-from itertools import chain
+from io import StringIO
+from itertools import chain, zip_longest
 from logging import LogRecord
 from typing import (
     TYPE_CHECKING,
@@ -221,7 +222,7 @@ class Case:  # pylint: disable=too-many-public-methods
         printed_kwargs = ", ".join(
             f"{key}={repr(value)}" for key, value in kwargs.items() if should_display(key, value)
         )
-        url = kwargs["url"].replace("'", "\\'")
+        url = _escape_single_quotes(kwargs["url"])
         args_repr = f"'{url}'"
         if printed_kwargs:
             args_repr += f", {printed_kwargs}"
@@ -374,7 +375,7 @@ class Case:  # pylint: disable=too-many-public-methods
     def call_asgi(
         self,
         app: Any = None,
-        base_url: Optional[str] = "http://testserver",
+        base_url: Optional[str] = None,
         headers: Optional[Dict[str, str]] = None,
         **kwargs: Any,
     ) -> requests.Response:
@@ -384,6 +385,8 @@ class Case:  # pylint: disable=too-many-public-methods
                 "ASGI application instance is required. "
                 "Please, set `app` argument in the schema constructor or pass it to `call_asgi`"
             )
+        if base_url is None:
+            base_url = self.get_full_base_url()
         client = ASGIClient(application)
 
         return self.call(base_url=base_url, session=client, headers=headers, **kwargs)
@@ -471,6 +474,37 @@ class Case:  # pylint: disable=too-many-public-methods
         )
 
 
+BACKSLASH = "\\"
+SINGLE_QUOTE = "'"
+ESCAPED_SINGLE_QUOTE = "\\'"
+
+
+def _escape_single_quotes(url: str) -> str:
+    """Escape single quotes in a string, so it is usable as in generated Python code.
+
+    The usual ``str.replace`` is not suitable as it may convert already escaped quotes to not-escaped.
+    """
+    output = StringIO()
+    url_chars = zip_longest(url, url[1:])
+    for pair in url_chars:
+        if pair == (BACKSLASH, BACKSLASH):
+            # Escaped backslash
+            output.write(BACKSLASH)
+            output.write(BACKSLASH)
+            next(url_chars)
+        elif pair == (BACKSLASH, SINGLE_QUOTE):
+            # Already escaped quote - write it and skip one char to avoid double-escaping
+            output.write(ESCAPED_SINGLE_QUOTE)
+            next(url_chars)
+        elif pair[0] == SINGLE_QUOTE:
+            # Escape single quote on first occurrence. It can't be already escaped one as it is handled above
+            output.write(ESCAPED_SINGLE_QUOTE)
+        else:
+            # Default case
+            output.write(pair[0])
+    return output.getvalue()
+
+
 @contextmanager
 def cookie_handler(client: werkzeug.Client, cookies: Optional[Cookies]) -> Generator[None, None, None]:
     """Set cookies required for a call."""
@@ -548,6 +582,10 @@ class APIOperation(Generic[P, C]):
     def links(self) -> Dict[str, Dict[str, Any]]:
         return self.schema.get_links(self)
 
+    def iter_parameters(self) -> Iterator[P]:
+        """Iterate over all operation's parameters."""
+        return chain(self.path_parameters, self.headers, self.cookies, self.query)
+
     def add_parameter(self, parameter: P) -> None:
         """Add a new processed parameter to an API operation.
 
