@@ -128,28 +128,43 @@ class BaseRunner:
             if isinstance(result, Ok):
                 operation, test = result.ok()
                 feedback = Feedback(self.stateful, operation)
-                for event in run_test(
-                    operation,
-                    test,
-                    results=results,
-                    feedback=feedback,
-                    recursion_level=recursion_level,
-                    data_generation_method=data_generation_method,
-                    **kwargs,
-                ):
-                    yield event
-                    if isinstance(event, events.Interrupted):
-                        return
-                # Additional tests, generated via the `feedback` instance
-                yield from self._run_tests(
-                    feedback.get_stateful_tests,
-                    template,
-                    settings,
-                    seed,
-                    recursion_level=recursion_level + 1,
-                    results=results,
-                    **kwargs,
-                )
+                # Track whether `BeforeExecution` was already emitted.
+                # Schema error may happen before / after `BeforeExecution`, but it should be emitted only once
+                # and the `AfterExecution` event should have the same correlation id as previous `BeforeExecution`
+                before_execution_correlation_id = None
+                try:
+                    for event in run_test(
+                        operation,
+                        test,
+                        results=results,
+                        feedback=feedback,
+                        recursion_level=recursion_level,
+                        data_generation_method=data_generation_method,
+                        **kwargs,
+                    ):
+                        yield event
+                        if isinstance(event, events.BeforeExecution):
+                            before_execution_correlation_id = event.correlation_id
+                        if isinstance(event, events.Interrupted):
+                            return
+                    # Additional tests, generated via the `feedback` instance
+                    yield from self._run_tests(
+                        feedback.get_stateful_tests,
+                        template,
+                        settings,
+                        seed,
+                        recursion_level=recursion_level + 1,
+                        results=results,
+                        **kwargs,
+                    )
+                except InvalidSchema as exc:
+                    yield from handle_schema_error(
+                        exc,
+                        results,
+                        data_generation_method,
+                        recursion_level,
+                        before_execution_correlation_id=before_execution_correlation_id,
+                    )
             else:
                 # Schema errors
                 yield from handle_schema_error(result.err(), results, data_generation_method, recursion_level)
@@ -185,13 +200,18 @@ class EventStream:
 
 
 def handle_schema_error(
-    error: InvalidSchema, results: TestResultSet, data_generation_method: DataGenerationMethod, recursion_level: int
+    error: InvalidSchema,
+    results: TestResultSet,
+    data_generation_method: DataGenerationMethod,
+    recursion_level: int,
+    *,
+    before_execution_correlation_id: Optional[str] = None,
 ) -> Generator[events.ExecutionEvent, None, None]:
     if error.method is not None:
         assert error.path is not None
         assert error.full_path is not None
         method = error.method.upper()
-        verbose_name = f"{method} {error.path}"
+        verbose_name = f"{method} {error.full_path}"
         result = TestResult(
             method=method,
             path=error.full_path,
@@ -199,16 +219,20 @@ def handle_schema_error(
             data_generation_method=data_generation_method,
         )
         result.add_error(error)
-        correlation_id = uuid.uuid4().hex
-        yield events.BeforeExecution(
-            method=method,
-            path=error.full_path,
-            verbose_name=verbose_name,
-            relative_path=error.path,
-            recursion_level=recursion_level,
-            data_generation_method=data_generation_method,
-            correlation_id=correlation_id,
-        )
+        # It might be already emitted - reuse its correlation id
+        if before_execution_correlation_id is not None:
+            correlation_id = before_execution_correlation_id
+        else:
+            correlation_id = uuid.uuid4().hex
+            yield events.BeforeExecution(
+                method=method,
+                path=error.full_path,
+                verbose_name=verbose_name,
+                relative_path=error.path,
+                recursion_level=recursion_level,
+                data_generation_method=data_generation_method,
+                correlation_id=correlation_id,
+            )
         yield events.AfterExecution(
             method=method,
             path=error.full_path,
@@ -223,7 +247,7 @@ def handle_schema_error(
         )
         results.append(result)
     else:
-        # When there is no `method`, then the schema error may cover multiple operations and we can't display it in
+        # When there is no `method`, then the schema error may cover multiple operations, and we can't display it in
         # the progress bar
         results.generic_errors.append(error)
 
@@ -340,7 +364,7 @@ def setup_hypothesis_database_key(test: Callable, operation: APIOperation) -> No
     # we use all API operation parameters in the digest.
     extra = operation.verbose_name.encode("utf8")
     for parameter in operation.definition.parameters:
-        extra += parameter.serialize().encode("utf8")
+        extra += parameter.serialize(operation).encode("utf8")
     test.hypothesis.inner_test._hypothesis_internal_add_digest = extra  # type: ignore
 
 
