@@ -29,66 +29,18 @@ class MutationResult(enum.Enum):
     SUCCESS = 1
     FAILURE = 2
 
-    @property
-    def is_success(self) -> bool:
-        return self == MutationResult.SUCCESS
-
-    @property
-    def is_failure(self) -> bool:
-        return self == MutationResult.FAILURE
-
     def __ior__(self, other: Any) -> "MutationResult":
         return self | other
 
     def __or__(self, other: Any) -> "MutationResult":
         # Syntactic sugar to simplify handling of multiple results
-        if not isinstance(other, MutationResult):
-            return NotImplemented
-        if self.is_success:
+        if self == MutationResult.SUCCESS:
             return self
         return other
 
 
 Mutation = Callable[["MutationContext", Draw, Schema], MutationResult]
-ALL_KEYWORDS = (
-    "additionalItems",
-    "additionalProperties",
-    "allOf",
-    "anyOf",
-    "const",
-    "contains",
-    "contentEncoding",
-    "contentMediaType",
-    "dependencies",
-    "enum",
-    "else",
-    "exclusiveMaximum",
-    "exclusiveMinimum",
-    "format",
-    "if",
-    "items",
-    "maxItems",
-    "maxLength",
-    "maxProperties",
-    "maximum",
-    "minItems",
-    "minLength",
-    "minProperties",
-    "minimum",
-    "multipleOf",
-    "not",
-    "oneOf",
-    "pattern",
-    "patternProperties",
-    "properties",
-    "propertyNames",
-    "$ref",
-    "required",
-    "then",
-    "type",
-    "uniqueItems",
-)
-ANY_TYPE_KEYS = ("$ref", "allOf", "anyOf", "const", "else", "enum", "if", "not", "oneOf", "then", "type")
+ANY_TYPE_KEYS = {"$ref", "allOf", "anyOf", "const", "else", "enum", "if", "not", "oneOf", "then", "type"}
 TYPE_SPECIFIC_KEYS = {
     "number": ("multipleOf", "maximum", "exclusiveMaximum", "minimum", "exclusiveMinimum"),
     "integer": ("multipleOf", "maximum", "exclusiveMaximum", "minimum", "exclusiveMinimum"),
@@ -112,7 +64,8 @@ class MutationContext:
     """Meta information about the current mutation state."""
 
     # The original schema
-    schema: Schema = attr.ib()
+    keywords: Schema = attr.ib()  # only keywords
+    non_keywords: Schema = attr.ib()  # everything else
     # Schema location within API operation (header, query, etc)
     location: str = attr.ib()
     # Payload media type, if available
@@ -150,19 +103,18 @@ class MutationContext:
             mutations = [change_properties]
         else:
             # Body can be of any type and does not have any specific type semantic.
-            mutations = draw(ordered(get_mutations(draw, self.schema)))
-        keywords, non_keywords = split_schema(self.schema)
+            mutations = draw(ordered(get_mutations(draw, self.keywords)))
         # Deep copy all keywords to avoid modifying the original schema
-        new_schema = deepcopy(keywords)
+        new_schema = deepcopy(self.keywords)
         enabled_mutations = draw(st.shared(FeatureStrategy(), key="mutations"))  # type: ignore
         result = MutationResult.FAILURE
         for mutation in mutations:
             if enabled_mutations.is_enabled(mutation.__name__):
                 result |= mutation(self, draw, new_schema)
-        if result.is_failure:
+        if result == MutationResult.FAILURE:
             # If we failed to apply anything, then reject the whole case
             reject()  # type: ignore
-        new_schema.update(non_keywords)
+        new_schema.update(self.non_keywords)
         if self.is_header_location:
             # All headers should have names that can be sent over network
             new_schema["propertyNames"] = {"type": "string", "format": "_header_name"}
@@ -186,20 +138,6 @@ class MutationContext:
         return new_schema
 
 
-def split_schema(schema: Schema) -> Tuple[Schema, Schema]:
-    """Split the schema into two parts.
-
-    The first one contains only validation JSON Schema keywords, the second one everything else.
-    """
-    keywords, non_keywords = {}, {}
-    for keyword, value in schema.items():
-        if keyword in ALL_KEYWORDS:
-            keywords[keyword] = value
-        else:
-            non_keywords[keyword] = value
-    return keywords, non_keywords
-
-
 def for_types(*allowed_types: str) -> Callable[[Mutation], Mutation]:
     """Immediately return FAILURE for schemas with types not from ``allowed_types``."""
 
@@ -329,7 +267,7 @@ def change_properties(context: MutationContext, draw: Draw, schema: Schema) -> M
     # one property
     ordered_properties = draw(ordered(properties, unique_by=lambda x: x[0]))
     for property_name, property_schema in ordered_properties:
-        if apply_until_success(context, draw, property_schema).is_success:
+        if apply_until_success(context, draw, property_schema) == MutationResult.SUCCESS:
             # It is still possible to generate "positive" cases, for example, when this property is optional.
             # They are filtered out on the upper level anyway, but to avoid performance penalty we adjust the schema
             # so the generated samples are less likely to be "positive"
@@ -361,7 +299,7 @@ def change_properties(context: MutationContext, draw: Draw, schema: Schema) -> M
 
 def apply_until_success(context: MutationContext, draw: Draw, schema: Schema) -> MutationResult:
     for mutation in get_mutations(draw, schema):
-        if mutation(context, draw, schema).is_success:
+        if mutation(context, draw, schema) == MutationResult.SUCCESS:
             return MutationResult.SUCCESS
     return MutationResult.FAILURE
 
@@ -387,7 +325,7 @@ def _change_items_object(context: MutationContext, draw: Draw, schema: Schema, i
     result = MutationResult.FAILURE
     for mutation in get_mutations(draw, items):
         result |= mutation(context, draw, items)
-    if result.is_failure:
+    if result == MutationResult.FAILURE:
         return MutationResult.FAILURE
     min_items = schema.get("minItems", 0)
     schema["minItems"] = max(min_items, 1)
@@ -400,7 +338,7 @@ def _change_items_array(context: MutationContext, draw: Draw, schema: Schema, it
         result = MutationResult.FAILURE
         for mutation in get_mutations(draw, item):
             result |= mutation(context, draw, item)
-        if result.is_success:
+        if result == MutationResult.SUCCESS:
             latest_success_index = idx
     if latest_success_index is None:
         return MutationResult.FAILURE
